#!/usr/bin/perl

#    figtexture.pl - Figmap drawing utility
#    Copyright (C) 1999 Michael Jung
#    e-mail: miju@phantasia.org
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

# Takes fig polygons as convex polygons and creates (a) little lines
# inside of them, Mountain specification on Harn maps; (b) little dots
# inside of them, Hill specification on Harn maps; (c) long lines with
# occasional grass on top inside of them, Swamp specification on Harn
# maps. Works on all closed and open polylines of an input file
# (stdin/stdout if unspecified) of a specified color and depth or all,
# if unspecified. The used polygons are removed. Leaves the rest of
# the fig file untouched. Conserves pen and fill colors, and depth of
# the used polygons.

### ARGS SECTION

$Infile = "-";
$Outfile = "-";
$Color = -2;
$Depth = -1;
$Terrain = 0;

while ($#ARGV >= 0) {
    $arg = shift(@ARGV);
    if ($arg eq "-i") { $Infile = shift(@ARGV); }
    elsif ($arg eq "-o") { $Outfile = shift(@ARGV); }
    elsif ($arg eq "-d") { $Depth = shift(@ARGV); }
    elsif ($arg eq "-c") { $Color = shift(@ARGV); }
    elsif ($arg eq "-mountains") { $Terrain = 1; }
    elsif ($arg eq "-hills") { $Terrain = 2; }
    elsif ($arg eq "-swamp") { $Terrain = 3; }
    elsif ($arg eq "-field") { $Terrain = 4; }
    elsif ($arg eq "-seed") { srand(shift(@ARGV)); }
    else {
        print "Usage: figtexture.pl -swamp|-mountains|-hills|-field [-help] ";
	print "[-i infile] [-o outfile] [-d depth] [-c color] [-seed seed]\n";
        print "-help = this text\n";
        print "   -i = input file\n";
        print "   -o = output file\n";
	print "   -d = depth restriction\n";
	print "   -c = color restriction\n";
	print "-seed = random seed\n";
	exit;
    }
}

die "Error: `-mountains',`-swamp', `-field' or `-hills' are mandatory options!" if (!$Terrain);

open(OUTDATA,">$Outfile");
open(INDATA,"<$Infile");

### SCAN FILE

while (<INDATA>) {
    if (m/^2 3/) {
	$hold = $_;
	($sub_type, $line_style, $thickness,
	 $pen_color, $fill_color, $depth,
	 $pen_style, $area_fill, $style_val,
	 $join_style, $cap_style, $radius,
	 $forward_arrow, $backward_arrow,
	 $npoints) =
	     m/^2 ([-]*\d+) ([-]*\d+) ([-]*\d+) ([-]*\d+) ([-]*\d+) ([-]*\d+) ([-]*\d+) ([-]*\d+) ([-]*\d+\.\d+) ([-]*\d+) ([-]*\d+) ([-]*\d+) ([-]*\d+) ([-]*\d+) ([-]*\d+)/;
	if ((($Depth eq -1) || ($Depth eq $depth)) &&
	    (($Color eq -2) || ($Color eq $pen_color))) {
### MAIN LOOP
	    @p = ();
	    while (<INDATA>) {
		if (!m/\t/) { last; }
		s/^[ \t\r\n]+//eg;
		s/[ \t\r\n]+/" "/eg;
		push @p,split(/ /);
	    }
	    
	    $xm = $ym = 270*45;
	    $xM = $yM = 0;
	    
	    foreach (0..$#p) {
		if ($_ % 2 == 0) {
		    if ($xm > $p[$_]) { $xm = $p[$_]; }
		    if ($ym > $p[$_+1]) { $ym = $p[$_+1]; }
		    if ($xM < $p[$_]) { $xM = $p[$_]; }
		    if ($yM < $p[$_+1]) { $yM = $p[$_+1]; }
		}
	    }
	    
# Mountains
	    if ($Terrain eq 1) {
		foreach my $xc (0..200) {
		    foreach my $yc (0..270) {
			if (($xc - $yc%2) % 3) {
			    my $rx = (($yc) % 2)/4 - (($yc+1) % 2)/4;
			    my $ry = 0;
			    my $x = ($xc+$rx)*45;
			    my $y = ($yc+$ry)*45;
			    my $s = ($xc%3) * (3*($yc%2) - 2) - 4*($yc%2) + 3;
			    if (($x >= $xm) && ($x <= $xM) && ($y >= $ym) && ($y <= $yM)) {
				my $j = 0;
				while ($j+3 <= $#p) {
				    last if (-($x-$p[$j])*($p[$j+3]-$p[$j+1])+
					     ($y-$p[$j+1])*($p[$j+2]-$p[$j]) < 0);
				    $j += 2;
				}
				if ($j+3 > $#p) {
				    $r = int rand(3);
				    $x2 = $x + $s*45,   $y2 = $y      if ($r == 0);
				    $x2 = $x - $s*45/2, $y2 = $y + 45 if ($r == 1);
				    $x2 = $x - $s*45/2, $y2 = $y - 45 if ($r == 2);
				    print OUTDATA "2 1 0 1 $pen_color $fill_color $depth ",
				    "0 -1 0.000 1 1 7 0 0 2\n\t";
				    printf OUTDATA "%d %d %d %d\n", $x, $y, $x2, $y2;
				}
			    }
			}
		    }
		}
	    }
# Hills
	    if ($Terrain eq 2) {
		foreach my $xc (0..200) {
		    foreach my $yc (0..270) {
			my $rx = rand(30)-15;
			my $ry = rand(30)-15;
			my $x = $xc*45+$rx;
			my $y = $yc*45+$ry;
			if (($x >= $xm) && ($x <= $xM) && ($y >= $ym) && ($y <= $yM)) {
			    my $j = 0;
			    while ($j+3 <= $#p) {
				if (-($x-$p[$j])*($p[$j+3]-$p[$j+1])+
				    ($y-$p[$j+1])*($p[$j+2]-$p[$j]) < 0)
				{
				    last;
				}
				$j += 2;
			    }
			    if ($j+3 > $#p) {
				print OUTDATA "1 3 0 1 $pen_color $fill_color $depth ",
				"0 20 0.000 1 0.0000 ";
				printf OUTDATA "%d %d 4 4 %d %d %d %d\n", $x, $y, $x, $y, $x+4, $y;
			    }
			}
		    }
		}
	    }
# Swamp
	    if ($Terrain eq 3) {
		foreach my $xc (0..100) {
		    foreach my $yc (0..90) {
			my $x = $xc*45*2;
			my $y = $yc*45*3;
			if (($x >= $xm) && ($x <= $xM) && ($y >= $ym) && ($y <= $yM)) {
			    my $j = 0;
			    while ($j+3 <= $#p) {
				if (-($x-$p[$j])*($p[$j+3]-$p[$j+1])+
				    ($y-$p[$j+1])*($p[$j+2]-$p[$j]) < 0)
				{
				    last;
				}
				$j += 2;
			    }
			    if ($j+3 > $#p) {
				if (rand(5) <= 4) {
				    print OUTDATA "2 1 0 1 $pen_color $fill_color $depth ",
				    "0 -1 0.000 1 1 7 0 0 2\n\t";
				    printf OUTDATA "%d %d %d %d\n",$x,$y,$x+90,$y;
				    if (rand(6) <= 1) {
					print OUTDATA "2 1 0 1 $pen_color $fill_color $depth ",
					"0 -1 0.000 1 1 7 0 0 2\n\t";
					printf OUTDATA "%d %d %d %d\n",$x+45,$y,$x+45,$y-45;
					print OUTDATA "2 1 0 1 $pen_color $fill_color $depth ",
					"0 -1 0.000 1 1 7 0 0 2\n\t";
					printf OUTDATA "%d %d %d %d\n",$x+55,$y,$x+85,$y-45;
					print OUTDATA "2 1 0 1 $pen_color $fill_color $depth ",
					"0 -1 0.000 1 1 7 0 0 2\n\t";
					printf OUTDATA "%d %d %d %d\n",$x+35,$y,$x+5,$y-45;
				    }
				}
			    }
			}
		    }
		}
	    }
# Field
	    if ($Terrain eq 4) {
		foreach my $xc (0..200) {
		    @pf = ();
		    foreach my $yc (0..810) {
			my $j = 0;
			my $x = $xc*45;
			my $y = $yc*15;
			if (($x >= $xm) && ($x <= $xM) && ($y >= $ym) && ($y <= $yM)) {
			    while ($j+3 <= $#p) {
				if (-($x-$p[$j])*($p[$j+3]-$p[$j+1])+
				    ($y-$p[$j+1])*($p[$j+2]-$p[$j]) < 0)
				{
				    last;
				}
				$j += 2;
			    }
			    if ($j+3 > $#p) {
				push @pf, ($x+int(rand(10)-5), $y);
			    }
			}
		    }
		    if ($#pf - 2 > 0) {
			print OUTDATA "2 3 $line_style 1 $pen_color $fill_color $depth ";
			print OUTDATA "$pen_style -1 0.000 1 1 -1 0 0 ",($#pf+1)/2;
			print OUTDATA "\n\t ",join(" ",@pf),"\n";
		    }
		}
	    }
	    redo;
### END MAIN LOOP
	}
	else {
	    print OUTDATA $hold;
	}
    }
    else {
	print OUTDATA;
    }
}

